def fwrite(path: str, text, encoding='cp949'):
    """
    It writes a text to a file
    
    :param path: The path to the file you want to write to
    :type path: str
    :param text: The text to be written
    :param encoding: The encoding of the file, defaults to cp949 (optional)
    """

    text = str(text)
    
    if not text:
        text += '\n'
    elif text[-1] != '\n':
        text += '\n'
    
    try:
        with open(path, 'a', encoding=encoding) as f:
            f.write(text)
    except UnicodeEncodeError:
        try:
            with open(path, 'a', encoding='cp949') as f:
                f.write(text)
        except UnicodeEncodeError:
            with open(path, 'a', encoding='utf-8') as f:
                f.write(text)


def distinct(myList: list):
    """
    It takes a list as an argument, converts it to a dictionary, and then returns the list of keys
    
    :param myList: list
    :type myList: list
    :return: A list of unique values from the list passed in.
    """
    return list(dict.fromkeys(myList))


def ip_change2():
    """Change IP.
    USB Tethering is Needed.

    Returns:
        bool: True if success, False otherwise.
    """
    import requests
    import subprocess
    try:
        old_ip = requests.get('https://api.myip.com').json()['ip']
    except:
        while True:
            try:
                old_ip = requests.get('https://api.myip.com').json()['ip']
            except:
                pass
            else:
                break
    subprocess.run(['c:\\adb\\adb', 'shell', 'am', 'start', '-a', 'android.intent.action.MAIN', '-n', 'com.mmaster.mmaster/com.mmaster.mmaster.MainActivity'])
    result_flag = False
    for cnt in range(90):
        print('인터넷 접속대기중 - {}초'.format(cnt+1))
        try:
            cur_ip = requests.get('https://api.myip.com', timeout = 1).json()['ip']
            if old_ip == cur_ip:
                print('아이피가 변경되지 않았습니다.')
                return result_flag
            else:
                print(f'{old_ip} -> {cur_ip} 변경 완료.')
                result_flag = True
                return result_flag
        except:
            pass
    print('아이피가 변경되지 않았습니다.')
    return result_flag


def http_remove(link: str):
    """
    It takes a string, removes the http:// or https:// from the beginning of the string, and then
    removes any trailing slashes
    
    :param link: str = The link you want to remove the http(s):// from
    :type link: str
    :return: The link without the http:// or https://
    """
    import re
    link = re.sub(r"https?://", '', link).strip('/')
    return link


def http_append(link: str):
    '''url에 https 추가'''
    return link if link.startswith('http') else "http://" + link


def get_code_from_image(img_path: str):
    """
    > The function takes an image path as an argument and returns the text in the image
    
    :param img_path: The path to the image file
    :type img_path: str
    :return: The return value is the text of the captcha.
    """
    import sys
    import datetime
    from python_anticaptcha import AnticaptchaClient, ImageToTextTask, AnticaptchaException
    
    ret_val = 'Failed'

    API_KEY = 'b336be7de932b65c877403893a382713'
    
    sys.stdout.write(f'> 보안코드 분석중 (이미지 기반) - {datetime.datetime.now().strftime("%H:%M:%S")}\n')

    try:
        img = open(img_path, 'rb')
    except:
        return ret_val
    
    client = AnticaptchaClient(API_KEY)
    task = ImageToTextTask(img)
    
    try:
        job = client.createTask(task)
        job.join()
    except AnticaptchaException:
        pass
    else:
        ret_val = job.get_captcha_text()
    
    return ret_val


def sound_alert(__msg: str='> 계속하려면 엔터를 입력하세요.\n> '):
    """
    > It plays a bell sound in a loop until the user presses the Enter key
    
    :param __msg: str='> 계속하려면 엔터를 입력하세요.\n> ', defaults to > 계속하려면 엔터를 입력하세요.\n> 
    :type __msg: str (optional)
    """
    import time
    import winsound
    import threading
    import pkgutil

    _pkg = '.'.join(__name__.split('.')[:-1])

    bell_data = pkgutil.get_data(_pkg, 'Bell.wav')

    class AlertThread(threading.Thread):
        
        def __init__(self):
            threading.Thread.__init__(self)
            self.flag = threading.Event()
        
        def run(self):

            while not self.flag.is_set():
                winsound.PlaySound(bell_data, winsound.SND_MEMORY)
                time.sleep(1)
    
    alert = AlertThread()

    alert.start()
    input(__msg)
    alert.flag.set()
    alert.join()



def run_everyday_between(start_time: str, end_time: str, job, *args, **kwargs):
    """
    It runs a function between two times every day
    
    :param start_time: str - The time you want the job to start running
    :type start_time: str
    :param end_time: The time you want the job to stop running
    :type end_time: str
    :param job: the function you want to run
    """
    import datetime
    start_hour, start_minute = [int(x.strip()) for x in start_time.split(':')]
    end_hour, end_minute = [int(x.strip()) for x in end_time.split(':')]
    
    while True:
        if datetime.time(start_hour, start_minute) <= datetime.datetime.now().time() <= datetime.time(end_hour, end_minute):
            job(*args, **kwargs)



class TelegramSender:

    import time
    import telegram
    from datetime import datetime
    from dateutil.parser import parse

    def __init__(self, token) -> None:
        self.token = token
        self.lastSendTime = None
        self.sendDelay = 12
        self.__bot = self.telegram.Bot(token)

    @property
    def lastSendTime(self):
        return self._lastSendTime
    
    @lastSendTime.setter
    def lastSendTime(self, value):
        if isinstance(value, str):
            try:
                self._lastSendTime = self.parse(value)
            except:
                self._lastSendTime = value
        elif isinstance(value, self.datetime):
            self._lastSendTime = value
        elif value is None:
            self._lastSendTime = value
        else:
            raise ValueError
    
    @property
    def token(self):
        return self._token
    
    @token.setter
    def token(self, value):
        if isinstance(value, str):
            self._token = value
        elif value is None:
            self._token = value
        else:
            raise ValueError
    
    @property
    def sendDelay(self):
        return self._sendDelay
    
    @sendDelay.setter
    def sendDelay(self, value):
        if isinstance(value, int):
            self._sendDelay = float(value)
        elif isinstance(value, float):
            self._sendDelay = value
        elif value is None:
            self._sendDelay = value
        else:
            raise ValueError

    def send(self, chat_id: str, text: str, **kwargs):
        try:
            if self.lastSendTime:
                total_diff = (self.datetime.now() - self.lastSendTime).total_seconds()
                if total_diff < self.sendDelay:
                    print(f'마지막 메시지 전송 후 {total_diff}초 입니다.\n{self.sendDelay-total_diff}초 후 메시지를 전송합니다.')
                    self.time.sleep(self.sendDelay-total_diff)
                    if kwargs:
                        msg = self.__bot.sendMessage(chat_id=chat_id, text=text, **kwargs)
                    else:
                        msg = self.__bot.sendMessage(chat_id=chat_id, text=text)

                    self.lastSendTime = self.datetime.now()
                else:
                    if kwargs:
                        msg = self.__bot.sendMessage(chat_id=chat_id, text=text, **kwargs)
                    else:
                        msg = self.__bot.sendMessage(chat_id=chat_id, text=text)
                    self.lastSendTime = self.datetime.now()

            else:
                if kwargs:
                    msg = self.__bot.sendMessage(chat_id=chat_id, text=text, **kwargs)
                else:
                    msg = self.__bot.sendMessage(chat_id=chat_id, text=text)
                self.lastSendTime = self.datetime.now()
        except:
            return self.send(chat_id=chat_id, text=text)
        else:
            return msg

    def send_photo(self, chat_id: str, img_path: str, **kwargs):
        try:
            if self.lastSendTime:
                total_diff = (self.datetime.now() - self.lastSendTime).total_seconds()
                if total_diff < self.sendDelay:
                    print(f'마지막 메시지 전송 후 {total_diff}초 입니다.\n{self.sendDelay-total_diff}초 후 메시지를 전송합니다.')
                    self.time.sleep(self.sendDelay-total_diff)
                    if kwargs:
                        msg = self.__bot.sendPhoto(chat_id=chat_id, photo=open(img_path,'rb'), **kwargs)
                    else:
                        msg = self.__bot.sendPhoto(chat_id=chat_id, photo=open(img_path,'rb'))
                    self.lastSendTime = self.datetime.now()
                else:
                    if kwargs:
                        msg = self.__bot.sendPhoto(chat_id=chat_id, photo=open(img_path,'rb'), **kwargs)
                    else:
                        msg = self.__bot.sendPhoto(chat_id=chat_id, photo=open(img_path,'rb'))
                    self.lastSendTime = self.datetime.now()

            else:
                if kwargs:
                    msg = self.__bot.sendPhoto(chat_id=chat_id, photo=open(img_path,'rb'), **kwargs)
                else:
                    msg = self.__bot.sendPhoto(chat_id=chat_id, photo=open(img_path,'rb'))
                self.lastSendTime = self.datetime.now()
        except:
            return self.send_photo(chat_id=chat_id, img_path=img_path)
        else:
            return msg



__all__ = ['selenium', 'paramiko', 'telegram', 'dropbox', 'util', 'schedule']

__version__ = '0.6.5'

from . import selenium, paramiko, telegram, dropbox, util, schedule

from .util import *

def is_latest_version() -> bool:
    """
    It checks if the current version of the package is the latest version
    :return: A boolean value.
    """
    import feedparser
    feed = feedparser.parse('https://pypi.org/rss/project/scode/releases.xml')
    latest_version = feed.entries[0]['title']
    return __version__ == latest_version


def update_scode() -> None:
    """
    It updates the PATH environment variable to include the Python 3.4 and 3.8 directories, then runs
    the command `python -m pip install -U scode` to update the `scode` package
    """
    import os
    import subprocess
    os.environ['PATH'] = ';'.join(distinct(os.getenv('PATH').split(';') if os.getenv('PATH') else [] + ["C:\\Python38\\Scripts\\", "C:\\Python38\\", "C:\\Python34\\Scripts\\", "C:\\Python34\\"]))
    subprocess.run(['python', '-m', 'pip', 'install', '-U', 'scode'])


def refresh() -> None:
    """
    It reloads the current module
    """
    import sys
    import importlib
    importlib.reload(sys.modules[__name__])

try:
    is_outdated = not is_latest_version()
except Exception as e:
    import warnings
    warnings.warn(f"{type(e).__name__}: Skipped auto update sequence because error occurred while checking the version", Warning)
else:
    if is_outdated:

        try:
            update_scode()
        except Exception as e:
            import warnings
            warnings.warn(f"{type(e).__name__}: Skipped auto update sequence because error occurred while updating", Warning)
        else:
            refresh()



import setuptools
from scode import __version__ as scode_version

with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="scode", # Replace with your own username
    version=scode_version,
    author="강동욱, 선준우, 강진영, 박태준, 김도현, 임성현",
    maintainer = '강동욱',
    maintainer_email="ehddnrwork@gmail.com",
    description="The Private Package of Showm Company.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/showm-dev/Standard",
    packages=setuptools.find_packages(),
    package_data={
        'scode': ['Bell.wav'],
    },
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Microsoft :: Windows",
    ],
    python_requires='>=3.6',
    install_requires=[
        'selenium==3.141.0',
        'paramiko',
        'dropbox',
        'telegram',
        'python-dateutil',
        'requests',
        'pyperclip',
        'chromedriver-autoinstaller',
        'fake-useragent',
        'anticaptchaofficial',
        'python_anticaptcha',
        'feedparser',
    ],
)



import time
import paramiko


def ssh_connect(hostname: str, username: str, password: str) -> paramiko.SSHClient:
    """
    > This function takes a hostname, username, and password as arguments and returns an SSHClient
    object
    
    :param hostname: The IP address or hostname of the server you want to connect to
    :type hostname: str
    :param username: The username to log into the server with
    :type username: str
    :param password: The password for the user you're logging in as
    :type password: str
    :return: A SSHClient object.
    """
    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    ssh.connect(hostname=hostname, port=22, username=username, password=password)
    return ssh


def command(ssh: paramiko.SSHClient, query: str, timeout: float = None):
    """
    It executes a command on a remote server and returns the output and error text
    
    :param ssh: paramiko.SSHClient
    :type ssh: paramiko.SSHClient
    :param query: The command you want to run on the remote server
    :type query: str
    :param timeout: The time in seconds to wait for the command to finish. If the command doesn't finish
    in the given time, the function will return the output of the command so far
    :type timeout: float
    :return: stdout_text, err_text
    """

    stdin, stdout, stderr = ssh.exec_command(query)
    
    if timeout:
        start_time = time.time()
    
    # Wait for the command to terminate  
    while not stdout.channel.exit_status_ready():
        time.sleep(1)
        if timeout:
            latest_time = time.time()

            if latest_time - start_time > timeout:
                stdout_text = stdout.read().decode('utf-8').strip()
                err_text = stderr.read().decode('utf-8').strip()
                return stdout_text, err_text
    
    stdout_text = stdout.read().decode('utf-8').strip()
    err_text = stderr.read().decode('utf-8').strip()
    return stdout_text, err_text


def execute_sql_query(ssh: paramiko.SSHClient, user_id: str, user_pw: str, db_name: str, query: str, timeout = None):
    """execute sql query

    Args:
        ssh (paramiko.SSHClient): paramiko ssh client.
        user_id (str): ID to log in.
        user_pw (str): Password to log in.
        db_name (str): Name of the database to be connected to.
        query (str): query statement to execute.

    Returns:
        tuple[str, str]: stdout, stderr
    """    
    query = query.replace('\'','\"')
    fquery = f"""mysql -u{user_id} -p{user_pw} {db_name} -e '{query}'"""
    return command(ssh, fquery, timeout=timeout)



def dropbox_upload_file(src_file_path, destination_path, ACCESS_TOKEN):
    """
    It takes a file path, uploads it to Dropbox, and returns the file path of the uploaded file
    
    :param src_file_path: The path to the file you want to upload
    :param destination_path: The path to the folder where you want to upload the file
    :param ACCESS_TOKEN: The access token you got from the Dropbox API console
    """
    import dropbox
    dbx = dropbox.Dropbox(ACCESS_TOKEN)
    destination_path = destination_path.replace('\\', '/')
    src_file_path = src_file_path.replace('\\', '/')
    if destination_path[-1] == '/':
        destination_path = destination_path[:-1]
    if '.' not in destination_path.split('/')[-1]:
        destination_path += "/" + src_file_path.split('/')[-1]
    with open(src_file_path, "rb") as f:
        dbx.files_upload(f.read(), destination_path, mode=dropbox.files.WriteMode.overwrite)

def dropbox_upload_folder(folder_path, destination_path, ACCESS_TOKEN):
    """
    It takes a folder path, a destination path, and an access token, and uploads the folder to the
    destination path on Dropbox
    
    :param folder_path: The path to the folder you want to upload
    :param destination_path: The path to the folder in Dropbox where you want to upload the file
    :param ACCESS_TOKEN: The access token for your Dropbox account
    """
    import os
    import glob
    folder_path = folder_path.replace('\\', '/')
    if folder_path[-1] == '/':
        folder_path = folder_path[:-1]
    destination_path = destination_path.replace('\\', '/')
    if destination_path[-1] == '/':
        destination_path = destination_path[:-1]
    file_list = glob.glob(f'{folder_path}/**')
    for file_path in file_list:
        filename = file_path.split('\\')[-1]
        if os.path.isdir(file_path):
            dropbox_upload_folder(file_path, f"{destination_path}/{filename}", ACCESS_TOKEN)
        else:
            dropbox_upload_file(file_path, f'{destination_path}/{filename}', ACCESS_TOKEN)



import os
import sys
import time
import pyperclip
import random
import chromedriver_autoinstaller as AutoChrome
from fake_useragent import UserAgent
from selenium import webdriver
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.select import Select
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.remote.command import Command
from selenium.webdriver.remote.webelement import WebElement
from selenium.common.exceptions import *
from selenium.webdriver.chrome.webdriver import WebDriver
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
from selenium.webdriver.common.action_chains import ActionChains


def get_status(driver: WebDriver):
    """
    If the driver can get the title of the page, it's alive, otherwise it's dead
    
    :param driver: The webdriver object
    :type driver: WebDriver
    :return: The status of the driver.
    """
    try:
        driver.title
    except:
        return "Dead"
    else:
        return "Alive"


def load_driver(chrome_options=None, mode=None, userId=None, port=9222, **kwargs) -> WebDriver:
    """
    It opens a Chrome browser with the specified options
    
    :param chrome_options: This is an instance of the ChromeOptions class. You can pass an existing
    instance of ChromeOptions to this parameter, or you can create a new instance of ChromeOptions
    :param mode: This is the mode you want to use. You can use multiple modes at once
    :param userId: This is the user ID that you want to use. This is used to create a cache folder for
    the user
    :param port: The port number that the Chrome browser will be running on, defaults to 9222 (optional)
    :return: A WebDriver object.
    """

    # userAgent = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/61.0.3163.100 Safari/537.36'
    # userAgentMo = 'Mozilla/5.0 (iPhone; CPU iPhone OS 13_2_3 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/13.0.3 Mobile/15E148 Safari/604.1'
    # userAgent = UserAgent().random

    valid_mode_lst = [
        'fast',
        'cache',
        'debug',
        'secret'
    ]

    chrome_ver = AutoChrome.get_chrome_version().split('.')[0]
    chrome_driver_path = 'C:\\chromedriver\\'

    if not os.path.isdir('c:\\chromedriver'):
        os.mkdir('c:\\chromedriver')
    
    if chrome_options is None:
        chrome_options = webdriver.ChromeOptions()
        chrome_options.add_argument('--window-position=850,0')
    
    chrome_options.add_argument('--disable-features=ChromeWhatsNewUI')
    chrome_options.add_experimental_option("excludeSwitches", ["enable-logging", "test-type"])
    
    if not mode:

        pass

    else:

        if isinstance(mode, str):

            md = mode
            
            if md == 'fast':

                chrome_options.add_argument('disable-infobars')
                chrome_options.add_argument('--disable-extensions')
                chrome_options.add_argument('--disable-gpu')
                prefs = {'profile.default_content_setting_values': {'cookies' : 2, 'images': 2, 'plugins' : 2, 'popups': 2, 'geolocation': 2, 'notifications' : 2, 'auto_select_certificate': 2, 'fullscreen' : 2, 'mouselock' : 2, 'mixed_script': 2, 'media_stream' : 2, 'media_stream_mic' : 2, 'media_stream_camera': 2, 'protocol_handlers' : 2, 'ppapi_broker' : 2, 'automatic_downloads': 2, 'midi_sysex' : 2, 'push_messaging' : 2, 'ssl_cert_decisions': 2, 'metro_switch_to_desktop' : 2, 'protected_media_identifier': 2, 'app_banner': 2, 'site_engagement' : 2, 'durable_storage' : 2}}
                chrome_options.add_experimental_option('prefs', prefs)
                
            if md == 'cache':
                
                userDataFolder = 'c:/cache/{}'.format(userId)
                chrome_options.add_argument('--user-data-dir=' + userDataFolder)
                chrome_options.add_argument('--disk-cache-dir=' + userDataFolder)
            
            if md == 'debug':
                import subprocess
                try:
                    subprocess.Popen(rf'C:\Program Files\Google\Chrome\Application\chrome.exe --remote-debugging-port={port} --user-data-dir="C:\chrometemp"')
                except:
                    subprocess.Popen(rf'C:\Program Files (x86)\Google\Chrome\Application\chrome.exe --remote-debugging-port={port} --user-data-dir="C:\chrometemp"')

                chrome_options = Options()                                                                    # 옵션객체 생성
                chrome_options.add_experimental_option("debuggerAddress", f"127.0.0.1:{port}")
            
            if md == 'secret':
                chrome_options.add_argument('--incognito')
            
            if md not in valid_mode_lst:
                modes_text = ", ".join(valid_mode_lst)
                raise ValueError(f"Invalid Value : {mode}\nPlease Use below instead of {mode}\n\n{modes_text}\n\nUsage : load_driver(mode=['{random.choice(valid_mode_lst)}'])")
        else:
            for md in mode:
        
                if md == 'fast':

                    chrome_options.add_argument('disable-infobars')
                    chrome_options.add_argument('--disable-extensions')
                    chrome_options.add_argument('--disable-gpu')
                    prefs = {'profile.default_content_setting_values': {'cookies' : 2, 'images': 2, 'plugins' : 2, 'popups': 2, 'geolocation': 2, 'notifications' : 2, 'auto_select_certificate': 2, 'fullscreen' : 2, 'mouselock' : 2, 'mixed_script': 2, 'media_stream' : 2, 'media_stream_mic' : 2, 'media_stream_camera': 2, 'protocol_handlers' : 2, 'ppapi_broker' : 2, 'automatic_downloads': 2, 'midi_sysex' : 2, 'push_messaging' : 2, 'ssl_cert_decisions': 2, 'metro_switch_to_desktop' : 2, 'protected_media_identifier': 2, 'app_banner': 2, 'site_engagement' : 2, 'durable_storage' : 2}}
                    chrome_options.add_experimental_option('prefs', prefs)
                    
                if md == 'cache':
                    
                    userDataFolder = 'c:/cache/{}'.format(userId)
                    chrome_options.add_argument('--user-data-dir=' + userDataFolder)
                    chrome_options.add_argument('--disk-cache-dir=' + userDataFolder)
                
                if md == 'debug':
                    import subprocess
                    try:
                        subprocess.Popen(rf'C:\Program Files\Google\Chrome\Application\chrome.exe --remote-debugging-port={port} --user-data-dir="C:\chrometemp"')
                    except:
                        subprocess.Popen(rf'C:\Program Files (x86)\Google\Chrome\Application\chrome.exe --remote-debugging-port={port} --user-data-dir="C:\chrometemp"')

                    chrome_options = Options()                                                                    # 옵션객체 생성
                    chrome_options.add_experimental_option("debuggerAddress", f"127.0.0.1:{port}")
                
                if md == 'secret':
                    chrome_options.add_argument('--incognito')
                
                if md not in valid_mode_lst:
                    modes_text = ", ".join(valid_mode_lst)
                    raise ValueError(f"Invalid Value : {mode}\nPlease Use below instead of {mode}\n\n{modes_text}\n\nUsage : load_driver(mode=['{random.choice(valid_mode_lst)}'])")
    
    if os.path.isfile(f'c:\\chromedriver\\{chrome_ver}\\chromedriver.exe'):
        driver = webdriver.Chrome(executable_path=f'c:\\chromedriver\\{chrome_ver}\\chromedriver.exe', options=chrome_options, **kwargs)
    else:
        driver = webdriver.Chrome(executable_path=AutoChrome.install(path=chrome_driver_path), options=chrome_options, **kwargs)

    while len(driver.window_handles) > 1:
        driver.close()
        driver.switch_to.window(driver.window_handles[0])

    return driver


def load_driver2(port=9222):
    import subprocess
    
    chrome_ver = AutoChrome.get_chrome_version().split('.')[0]
    chrome_driver_path = 'C:\\chromedriver\\'

    if not os.path.isdir('c:\\chromedriver'):
        os.mkdir('c:\\chromedriver')

    # userAgent = UserAgent().random
    try:
        subprocess.Popen(rf'C:\Program Files\Google\Chrome\Application\chrome.exe --remote-debugging-port={port} --user-data-dir="C:\chrometemp"')
    except:
        subprocess.Popen(rf'C:\Program Files (x86)\Google\Chrome\Application\chrome.exe --remote-debugging-port={port} --user-data-dir="C:\chrometemp"')

    # d = DesiredCapabilities.CHROME
    # d['goog:loggingPrefs'] = { 'browser':'ALL' }
    chrome_options = Options()                                                                    # 옵션객체 생성
    chrome_options.add_experimental_option("debuggerAddress", f"127.0.0.1:{port}")
    # chrome_options.add_argument(f"user-agent={userAgent}")
    try:
        driver = webdriver.Chrome(executable_path=f'c:\\chromedriver\\{chrome_ver}\\chromedriver.exe', options=chrome_options)
    except:
        AutoChrome.install(path=chrome_driver_path)
        driver = webdriver.Chrome(executable_path=f'c:\\chromedriver\\{chrome_ver}\\chromedriver.exe', options=chrome_options)
    
    if len(driver.window_handles) > 1:
        driver.close()
        driver.switch_to.window(driver.window_handles[0])

    return driver


def load_cache_driver(userId, chrome_options = None, **kwargs):
    import warnings
    warnings.warn("load_cache_driver is deprecated. Use load_driver instead.", DeprecationWarning)
    return load_driver(chrome_options=chrome_options, mode=['cache'], userId=userId, **kwargs)


def n_login(driver: WebDriver, nid: str, pwd: str, keep_login: bool=True, ip_safe: bool=False, force: bool=False, mobile: bool=False, as_tuple: tuple=False):
    """
    It logs in to Naver.com and returns True if it was successful, False if it wasn't, and a string if
    there was a problem
    
    :param driver: WebDriver
    :type driver: WebDriver
    :param nid: Naver ID
    :type nid: str
    :param pwd: str
    :type pwd: str
    :param keep_login: If True, the account will be kept logged in. If False, the account will be logged
    out after closing the browser, defaults to True
    :type keep_login: bool (optional)
    :param ip_safe: If True, the login will be done with IP Safe, defaults to False
    :type ip_safe: bool (optional)
    :param force: If True, it will try to login even if it's already logged in, defaults to False
    :type force: bool (optional)
    :param mobile: bool=False, defaults to False
    :type mobile: bool (optional)
    :param as_tuple: If True, returns a tuple of (bool, str), defaults to False
    :type as_tuple: tuple (optional)
    :return: A tuple of two values.
    """
    
    flag = None
    
    try:
        
        if mobile:
            
            if force:
                pass
            else:
                
                try:
                    driver.get('https://m.naver.com/aside/')
                except TimeoutException:
                    try:
                        driver.refresh()
                    except TimeoutException:
                        sys.stdout.write('네트워크 문제로 로그인 되어있는지 확인이 불가합니다.\n')
                        flag = "네트워크 문제"
                        return
                except Exception as e:
                    sys.stdout.write(f'{e}\n')
                    flag = ' '.join(str(e).splitlines())
                    return
                
                time.sleep(.5)
                
                try:
                    if driver.find_element(By.CSS_SELECTOR, '.MM_LOGINOUT').text == '로그아웃':
                        flag = True
                        return
                except:
                    pass
            
            driver.header_overrides = {'Referer': 'http://m.naver.com/aside/'}
            driver.get('https://nid.naver.com/nidlogin.login?svctype=262144&amp;url=http://m.naver.com/aside/')

            origin_clip = pyperclip.paste()
            try:
                pyperclip.copy(nid)
                driver.find_elements(By.CSS_SELECTOR, '#id')[0].send_keys(Keys.CONTROL, 'v')
                time.sleep(0.5)

                pyperclip.copy(pwd)
                driver.find_elements(By.CSS_SELECTOR, '#pw')[0].send_keys(Keys.CONTROL, 'v')
            finally:
                pyperclip.copy(origin_clip)

            if keep_login:
                if driver.find_element(By.CSS_SELECTOR, 'input#stay').get_attribute('value').lower() == 'off':
                    driver.execute_script('arguments[0].click();', driver.find_element(By.CSS_SELECTOR, 'input#stay'))
            else:
                if driver.find_element(By.CSS_SELECTOR, 'input#stay').get_attribute('value').lower() == 'on':
                    driver.execute_script('arguments[0].click();', driver.find_element(By.CSS_SELECTOR, 'input#stay'))

            driver.find_elements(By.CSS_SELECTOR, 'button[class="btn_check"]')[0].click()
            time.sleep(1)
        
        else:
            
            if force:
                pass
            else:
                try:
                    driver.get('https://www.naver.com')
                except TimeoutException:
                    try:
                        driver.refresh()
                    except TimeoutException:
                        sys.stdout.write('네트워크 문제로 로그인 되어있는지 확인이 불가합니다.\n')
                        flag = "네트워크 문제"
                        return
                except Exception as e:
                    sys.stdout.write(f'{e}\n')
                    flag = ' '.join(str(e).splitlines())
                    return
                time.sleep(.5)
                try:
                    logout_iframe = driver.find_element(By.CSS_SELECTOR, 'iframe#minime')
                    driver.switch_to.frame(logout_iframe)
                    driver.find_element(By.CSS_SELECTOR, 'a.btn_logout')
                except:
                    pass
                else:
                    try:
                        sys.stdout.write(f'''{driver.find_element(By.CSS_SELECTOR, 'div.email.MY_EMAIL').text.split('@')[0]} : 이미 로그인 되어있습니다.\n''')
                    except:
                        pass
                    finally:
                        flag = True
                        return
                finally:
                    driver.switch_to.default_content()

            # driver.header_overrides = {'Referer': 'https://www.naver.com/', 'User-Agent': 'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.0.0 Safari/537.36'}
            driver.header_overrides = {'Referer': 'https://www.naver.com/'}
            driver.get('https://nid.naver.com/nidlogin.login?mode=form&url=https%3A%2F%2Fwww.naver.com')
            
            time.sleep(.5)

            origin_clip = pyperclip.paste()
            try:
                pyperclip.copy(nid)
                driver.find_element(By.CSS_SELECTOR, 'input#id').send_keys(Keys.CONTROL, 'v')
                time.sleep(.5)
                pyperclip.copy(pwd)
                driver.find_element(By.CSS_SELECTOR, 'input#pw').send_keys(Keys.CONTROL, 'v')
            finally:
                pyperclip.copy(origin_clip)
            
            if keep_login:
                if driver.find_element(By.CSS_SELECTOR, 'input#keep').get_attribute('value').lower() == 'off':
                    driver.execute_script('arguments[0].click();', driver.find_element(By.CSS_SELECTOR, 'input#keep'))
            else:
                if driver.find_element(By.CSS_SELECTOR, 'input#keep').get_attribute('value').lower() == 'on':
                    driver.execute_script('arguments[0].click();', driver.find_element(By.CSS_SELECTOR, 'input#keep'))

            time.sleep(1)

            if ip_safe:
                if driver.find_element(By.CSS_SELECTOR, 'input#smart_LEVEL').get_attribute('value') == '-1':
                    driver.execute_script('arguments[0].click();', driver.find_element(By.CSS_SELECTOR, 'input#switch'))
            else:
                if driver.find_element(By.CSS_SELECTOR, 'input#smart_LEVEL').get_attribute('value') == '1':
                    driver.execute_script('arguments[0].click();', driver.find_element(By.CSS_SELECTOR, 'input#switch'))
            
            time.sleep(1)

            # 로그인 버튼 클릭
            try:
                driver.find_element(By.CSS_SELECTOR, 'button[type="submit"][id="log.login"]').click()
            except:
                driver.execute_script('arguments[0].click();', driver.find_element(By.CSS_SELECTOR, 'button[type="submit"][id="log.login"]'))

        try:
            WebDriverWait(driver, 2.5).until(expected_conditions.staleness_of(driver.find_element(By.TAG_NAME, 'html')))
        except:
            pass
        try:
            WebDriverWait(driver, 2.5).until(expected_conditions.presence_of_element_located((By.TAG_NAME, 'html')))
        except:
            pass

        now_url = driver.current_url
        source = driver.page_source

        if 'https://m.naver.com/aside/' == now_url:
            flag = True
        
        elif 'https://www.naver.com/' == now_url:
            flag = True
        
        elif 'https://m.naver.com/' == now_url:
            flag = True
        
        elif 'idRelease' in now_url and '대량생성' in source:
            flag = '로그인제한(대량생성ID)'

        elif 'sleepId' in now_url and '회원님의 아이디는 휴면 상태로 전환되었습니다.' in source:
            flag = '휴면'
        
        elif 'https://nid.naver.com/nidlogin.login' == now_url and '가입하지 않은 아이디이거나, 잘못된 비밀번호입니다.' in source:
            flag = False
        
        elif 'https://nid.naver.com/nidlogin.login' == now_url and '스팸성 홍보활동' in source:
            flag = '보호조치(스팸성 홍보활동)'
        
        elif 'idSafetyRelease' in now_url and '회원님의 아이디를 보호하고 있습니다.' in source and '개인정보보호 및 도용' in source:
            flag = '보호조치(개인정보보호및도용)'
        
        elif 'idSafetyRelease' in now_url and '회원님의 아이디를 보호하고 있습니다.' in source and '타인으로 의심' in source:
            flag = '보호조치(타인의심)'
        
        elif 'idSafetyRelease' in now_url and '회원님의 아이디를 보호하고 있습니다.' in source:
            flag = '보호조치'

        elif 'https://nid.naver.com/user2/help/contactInfo?m=viewPhoneInfo' == now_url and '회원정보에 사용할 휴대 전화번호를 확인해 주세요.' in source:
            flag = True

        elif 'deviceConfirm' in now_url and '새로운 기기(브라우저) 로그인' in source:
            flag = True

        else:
            flag = False
        
        if flag is not True and flag is not False:
            import requests
            requests.get(f'http://aaa.e-e.kr/problemid/insert.php?id={nid}&desc={flag}')
    finally:
        
        if as_tuple:
            if flag == True:
                return True, "로그인 성공"
            
            elif isinstance(flag, str):
                return False, flag
            
            else:
                return False, "로그인 실패"
        else:
            return flag


def daum_mail_login(driver, did, pwd):

    try:
        driver.get('https://logins.daum.net/accounts/signinform.do?url=https%3A%2F%2Fmail.daum.net%2F')
    except:
        driver.refresh()
    driver.implicitly_wait(5)
    time.sleep(1)

    pyperclip.copy(did)
    driver.find_element(By.CSS_SELECTOR, 'input[type="email"]').send_keys(Keys.CONTROL + 'v')
    pyperclip.copy(pwd)
    driver.find_element(By.CSS_SELECTOR, 'input[type="password"]').send_keys(Keys.CONTROL + 'v')
    time.sleep(.5)
    driver.find_element(By.CSS_SELECTOR, 'label[class="lab_check"]').click()
    time.sleep(.5)
    driver.find_element(By.CSS_SELECTOR, 'button[id="loginBtn"]').click()
    driver.implicitly_wait(5)
    time.sleep(1)

    now_url = driver.current_url

    # 카카오 통합계정이면 
    if now_url == 'https://logins.daum.net/accounts/login.do?slevel=1':

        # 로그인 창
        driver.get('https://accounts.kakao.com/login?continue=https%3A%2F%2Flogins.daum.net%2Faccounts%2Fksso.do%3Frescue%3Dtrue%26url%3Dhttps%253A%252F%252Fmail.daum.net%252F')
        driver.implicitly_wait(3)
        time.sleep(1)

        # 아이디 비밀번호 입력
        pyperclip.copy(did)
        driver.find_element(By.CSS_SELECTOR, 'input[validator="email_or_phone_or_kakaoid"]').send_keys(Keys.CONTROL + 'v')
        pyperclip.copy(pwd)
        driver.find_element(By.CSS_SELECTOR, 'input[validator="password"]').send_keys(Keys.CONTROL + 'v')
        time.sleep(.5)
        driver.execute_script(""" document.querySelector('input[name="stay_signed_in"]').click() """)
        time.sleep(.5)
        driver.find_element(By.CSS_SELECTOR, 'button[class="btn_g btn_confirm submit"]').click()
        driver.implicitly_wait(5)

        # 메일쓰기 버튼 나올때까지 대기
        WebDriverWait(driver, 10).until(expected_conditions.presence_of_element_located((By.CSS_SELECTOR, 'button[class="btn_comm btn_write"]')))
        time.sleep(1)

    else:

        try:
            WebDriverWait(driver, 10).until(expected_conditions.presence_of_element_located((By.CSS_SELECTOR, 'button[class="btn_comm btn_write"]')))
            time.sleep(1)
        except:
            driver.find_element(By.CSS_SELECTOR, 'a[id="afterBtn"]').click()
            driver.implicitly_wait(10)
            time.sleep(2)


def t_login(driver, tid, pwd):
    try:
        driver.find_element(By.CSS_SELECTOR, 'a[href="/login"]').click()
        driver.implicitly_wait(5)
        time.sleep(.5)

        pyperclip.copy(tid)
        driver.find_element(By.CSS_SELECTOR, 'input[type="text"]').send_keys(Keys.CONTROL, 'v')
        time.sleep(.5)
        pyperclip.copy(pwd)
        driver.find_element(By.CSS_SELECTOR, 'input[type="password"]').send_keys(Keys.CONTROL, 'v')
        time.sleep(.5)

        driver.find_element(By.CSS_SELECTOR, 'div[role="button"]').click()
        driver.implicitly_wait(5)
        time.sleep(.5)
    except:
        print('로그인이 되어 있습니다.')
    driver.get('https://twitter.com')
    time.sleep(1)
    if driver.current_url == 'https://twitter.com/home':
        return True
    else:
        return False


def line_login(driver: WebDriver, line_email: str, line_passwd: str):
    login_flag = False
    driver.implicitly_wait(3)
    driver.get('https://m.naver.com/aside/')
    if driver.find_element(By.CSS_SELECTOR, 'a.MM_LOGINOUT').text == '로그아웃':
        login_flag = True
        # print(1)
        return login_flag

    try:
        driver.get('https://access.line.me/oauth2/v2.1/noauto-login?loginState=RqKRHGGcrATtnP3SI7gc3I&loginChannelId=1426360231&returnUri=%2Foauth2%2Fv2.1%2Fauthorize%2Fconsent%3Fscope%3Dprofile%2Bfriends%2Bmessage.write%2Btimeline.post%2Bphone%2Bemail%2Bopenid%26response_type%3Dcode%26redirect_uri%3Dhttps%253A%252F%252Fnid.naver.com%252Foauth%252Fglobal%252FlineCallback%26state%3D0120522838%26client_id%3D1426360231#/')
    except WebDriverException:
        login_flag = '라인 로그인창 접속 실패'
        return login_flag
    
    try:
        email_input_el = WebDriverWait(driver, 3).until(expected_conditions.presence_of_element_located((By.CSS_SELECTOR, 'input[name="tid"]')))	
        # print(2)
    except:
        try:
            c_bt_el = driver.find_element(By.CSS_SELECTOR, 'button.c-button.l-btn.c-button--allow')
            driver.execute_script("arguments[0].click();", c_bt_el)
            try:
                WebDriverWait(driver, 1.5).until(expected_conditions.alert_is_present())
            except:
                now_url = driver.current_url
                source = driver.page_source
                
                if 'https://www.naver.com/' in driver.current_url:
                    login_flag = True
                    # print(32)
                    return login_flag
                
                elif 'idRelease' in now_url and '대량생성' in source:
                    login_flag = '로그인제한(대량생성ID)'
                    return login_flag

                elif 'sleepId' in now_url and '회원님의 아이디는 휴면 상태로 전환되었습니다.' in source:
                    login_flag = '휴면'
                    return login_flag
                
                elif 'https://nid.naver.com/nidlogin.login' == now_url and '가입하지 않은 아이디이거나, 잘못된 비밀번호입니다.' in source:
                    login_flag = False
                    return login_flag
                
                elif 'https://nid.naver.com/nidlogin.login' == now_url and '스팸성 홍보활동' in source:
                    login_flag = '보호조치(스팸성 홍보활동)'
                    return login_flag
                
                elif 'idSafetyRelease' in now_url and '회원님의 아이디를 보호하고 있습니다.' in source and '개인정보보호 및 도용' in source:
                    login_flag = '보호조치(개인정보보호및도용)'
                    return login_flag
                
                elif 'idSafetyRelease' in now_url and '회원님의 아이디를 보호하고 있습니다.' in source and '타인으로 의심' in source:
                    login_flag = '보호조치(타인의심)'
                    return login_flag
                
                elif 'idSafetyRelease' in now_url and '회원님의 아이디를 보호하고 있습니다.' in source:
                    login_flag = '보호조치'
                    return login_flag

                elif 'https://nid.naver.com/user2/help/contactInfo?m=viewPhoneInfo' == now_url and '회원정보에 사용할 휴대 전화번호를 확인해 주세요.' in source:
                    login_flag = True
                    return login_flag

                elif 'deviceConfirm' in now_url and '새로운 기기(브라우저) 로그인' in source:
                    login_flag = True
                    return login_flag
                
                else:
                    login_flag = 'Unknown Error'
                    return login_flag
            else:
                try:
                    fail_alert = driver.switch_to.alert
                    if '정상 처리가 안되었음' in fail_alert.text:
                        login_flag = fail_alert.text
                        fail_alert.dismiss()
                    else:
                        login_flag = fail_alert.text
                        fail_alert.dismiss()
                        print(login_flag)
                    return login_flag
                except:
                    login_flag = 'Unknown Error'
                    return login_flag
            
            # driver.find_elements(By.CSS_SELECTOR, 'button.c-button.l-btn.c-button--allow')[0].click()
            # print(3)
        except:
            print('이메일 입력창을 찾을 수 없습니다. 담당자에게 문의해주세요.')
            # print(4)
            login_flag = '아이디 창 오류'
            return login_flag
    else:
        try:
            password_input_el = WebDriverWait(driver, 3).until(expected_conditions.presence_of_element_located((By.CSS_SELECTOR, 'input[name="tpasswd"]')))	
            # print(5)
        except:
            print('패스워드 입력창을 찾을 수 없습니다. 담당자에게 문의해주세요.')
            login_flag = '패스워드 창 오류'
            return login_flag
        
        else:
            email_input_el.send_keys(line_email)
            time.sleep(1)
            password_input_el.send_keys(line_passwd)
            time.sleep(1)
            submit_bt_el = driver.find_element(By.CSS_SELECTOR, 'button[type="submit"]')
            try:
                driver.execute_script("arguments[0].click();", submit_bt_el)
            except:
                print('패스워드 입력창을 찾을 수 없습니다. 담당자에게 문의해주세요.')
                login_flag = '패스워드 창 오류'
                return login_flag
            else:
                time.sleep(3)
            
            try:
                WebDriverWait(driver, 1.5).until(expected_conditions.alert_is_present())
            except:
                pass
            else:
                try:
                    fail_alert = driver.switch_to.alert
                    if '정상 처리가 안되었음' in fail_alert.text:
                        login_flag = fail_alert.text
                        fail_alert.dismiss()
                    else:
                        login_flag = fail_alert.text
                        fail_alert.dismiss()
                        print(login_flag)
                    return login_flag
                except:
                    login_flag = 'Unknown Error'
                    return login_flag
            
            # print(6)
            try:
                if '필요한 권한' in driver.page_source:
                    # print(7)
                    try:
                        driver.find_element(By.CSS_SELECTOR, 'button[type="submit"][class="c-button l-btn c-button--allow"]').click()
                        # print(8)
                    except:
                        auth_submit_button = driver.find_element(By.CSS_SELECTOR, 'button[type="submit"][class="c-button l-btn c-button--allow"]')
                        driver.execute_script("arguments[0].click();", auth_submit_button)
                        # print(9)
                    finally:
                        auth_try_cnt = 0
                        while login_flag == False:
                            # print(10)
                            if auth_try_cnt > 2:
                                login_flag = alert_text
                                # print(11)
                                return login_flag
                            try:
                                # print(12)
                                WebDriverWait(driver, 1.5).until(expected_conditions.alert_is_present())
                                auth_alert = driver.switch_to.alert
                                # print(13)
                            except:
                                # print(14)
                                time.sleep(1.5)
                                if 'naver.com' in driver.current_url:
                                    # print(15)
                                    login_flag = True
                                    return login_flag
                                else:
                                    # print(16)
                                    auth_try_cnt += 1
                                    continue
                            else:
                                time.sleep(1.5)
                                # print(17)
                                if '정상 처리가 안되었음' in auth_alert.text:
                                    alert_text = auth_alert.text
                                    # print(18)
                                    auth_alert.dismiss()
                                    time.sleep(1)
                                    try:
                                        driver.find_element(By.CSS_SELECTOR, 'button[type="submit"][class="c-button l-btn c-button--allow"]').click()
                                        # print(19)
                                    except:
                                        auth_submit_button = driver.find_element(By.CSS_SELECTOR, 'button[type="submit"][class="c-button l-btn c-button--allow"]')
                                        driver.execute_script("arguments[0].click();", auth_submit_button)
                                        # print(20)
                                    finally:
                                        time.sleep(1)
                                        # print(21)
                                        if 'naver.com' in driver.current_url:
                                            login_flag = True
                                            # print(22)
                                            return login_flag
                                else:
                                    alert_text = auth_alert.text
                                    # print(23)
                                    auth_alert.dismiss()
                                    time.sleep(1)
                                    # print(24)
                                    try:
                                        driver.find_element(By.CSS_SELECTOR, 'button[type="submit"][class="c-button l-btn c-button--allow"]').click()
                                        # print(25)
                                    except:
                                        # print(26)
                                        auth_submit_button = driver.find_element(By.CSS_SELECTOR, 'button[type="submit"][class="c-button l-btn c-button--allow"]')
                                        driver.execute_script("arguments[0].click();", auth_submit_button)
                                        # print(27)
                                    finally:
                                        # print(28)
                                        time.sleep(1)
                                        if 'naver.com' in driver.current_url:
                                            login_flag = True
                                            # print(29)
                                            return login_flag
                                auth_try_cnt += 1
                                # print(30)
                                continue
                else:
                    now_url = driver.current_url
                    source = driver.page_source
                    
                    if 'https://www.naver.com/' in driver.current_url:
                        login_flag = True
                        # print(32)
                        return login_flag
                    
                    elif 'idRelease' in now_url and '대량생성' in source:
                        login_flag = '로그인제한(대량생성ID)'
                        return login_flag

                    elif 'sleepId' in now_url and '회원님의 아이디는 휴면 상태로 전환되었습니다.' in source:
                        login_flag = '휴면'
                        return login_flag
                    
                    elif 'https://nid.naver.com/nidlogin.login' == now_url and '가입하지 않은 아이디이거나, 잘못된 비밀번호입니다.' in source:
                        login_flag = '잘못입력'
                        return login_flag
                    
                    elif 'https://nid.naver.com/nidlogin.login' == now_url and '스팸성 홍보활동' in source:
                        login_flag = '보호조치(스팸성 홍보활동)'
                        return login_flag
                    
                    elif 'idSafetyRelease' in now_url and '회원님의 아이디를 보호하고 있습니다.' in source and '개인정보보호 및 도용' in source:
                        login_flag = '보호조치(개인정보보호및도용)'
                        return login_flag
                    
                    elif 'idSafetyRelease' in now_url and '회원님의 아이디를 보호하고 있습니다.' in source and '타인으로 의심' in source:
                        login_flag = '보호조치(타인의심)'
                        return login_flag
                    
                    elif 'idSafetyRelease' in now_url and '회원님의 아이디를 보호하고 있습니다.' in source:
                        login_flag = '보호조치'
                        return login_flag

                    elif 'https://nid.naver.com/user2/help/contactInfo?m=viewPhoneInfo' == now_url and '회원정보에 사용할 휴대 전화번호를 확인해 주세요.' in source:
                        login_flag = True
                        return login_flag
                    elif 'deviceConfirm' in now_url and '새로운 기기(브라우저) 로그인' in source:
                        login_flag = True
                        return login_flag
                    else:
                        login_flag = 'Unknown Error'
                        return login_flag
                    
            except Exception as e:
                login_flag = 'Unknown Error'
                return login_flag
    return login_flag


def scrollDownUntilPageEnd(driver, SCROLL_PAUSE_SEC = 1):
    """
    It scrolls down the page until the page height is the same as the last scroll height
    
    :param driver: the webdriver object
    :param SCROLL_PAUSE_SEC: The number of seconds to wait between each scroll, defaults to 1 (optional)
    :return: The return value is the height of the document in pixels.
    """
    
    # 스크롤 높이 가져옴
    try:
        last_height = driver.execute_script("return document.body.scrollHeight")
    except:
        return False
    while True:
        # 끝까지 스크롤 다운
        driver.execute_script("window.scrollTo(0, document.body.scrollHeight);")

        # 1초 대기
        time.sleep(SCROLL_PAUSE_SEC)

        # 스크롤 다운 후 스크롤 높이 다시 가져옴
        new_height = driver.execute_script("return document.body.scrollHeight")
        if new_height == last_height:
            # 끝까지 스크롤 다운
            driver.execute_script("window.scrollTo(0, document.body.scrollHeight);")

            # 1초 대기
            time.sleep(SCROLL_PAUSE_SEC)

            # 스크롤 다운 후 스크롤 높이 다시 가져옴
            new_height = driver.execute_script("return document.body.scrollHeight")
            if new_height == last_height:
                break
            break
        last_height = new_height
    return True


def solve_reCAPTCHA(driver: WebDriver):
    """
    It takes a webdriver as an argument, finds the reCAPTCHA key and the current URL, and then uses the
    anticaptcha API to solve the reCAPTCHA
    
    :param driver: WebDriver
    :type driver: WebDriver
    :return: The return value is the result of the last expression in the function body, if any. If the
    function doesn't have a return statement, the function returns None.
    """
    
    from anticaptchaofficial.recaptchav2proxyless import recaptchaV2Proxyless

    try:
        target_site_url = driver.current_url
        target_site_key = driver.execute_script('''return document.querySelector('[data-sitekey]').getAttribute('data-sitekey');''')
    except:
        print('해당 사이트의 reCAPTCHA Key를 찾을 수 없습니다.')
        return False

    API_KEY = 'b336be7de932b65c877403893a382713'

    solver = recaptchaV2Proxyless()
    solver.set_verbose(1)
    solver.set_key(API_KEY)
    solver.set_website_url(target_site_url)
    solver.set_website_key(target_site_key)
    #set optional custom parameter which Google made for their search page Recaptcha v2
    #solver.set_data_s('"data-s" token from Google Search results "protection"')

    g_response = solver.solve_and_return_solution()
    if g_response != 0:
        # print("g-response: "+g_response)
        pass
    else:
        print("task finished with error "+solver.error_code)
    
    result_flag = driver.execute_script(f'''document.getElementById("g-recaptcha-response").innerHTML = "{g_response}"''')

    return result_flag == None
